"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.queryGetItem = exports.queryGetTableItem = void 0;
const query_1 = require("./query");
const utils_1 = require("../util/utils");
const Table_1 = require("../Table");
const queryGetTableItem = async (Table, index, key, input, dkClient = Table.dkClient) => {
    var _a;
    const setIndex = index !== Table_1.primaryIndex ? index : undefined;
    const hashKey = Table.config.indexes[index].hash.key;
    const sortKey = (_a = Table.config.indexes[index].sort) === null || _a === void 0 ? void 0 : _a.key;
    const output = await (0, utils_1.run)(async () => {
        if (sortKey) {
            return (0, query_1.queryTableItems)(Table, {
                ...input,
                index: setIndex,
                pageLimit: 1,
                keyConditionExpression: `${hashKey} = :hashValue AND ${sortKey} = :sortValue`,
                expressionAttributeValues: {
                    [`:hashValue`]: key[hashKey],
                    [`:sortValue`]: key[sortKey],
                    ...input === null || input === void 0 ? void 0 : input.expressionAttributeValues
                }
            }, dkClient);
        }
        return (0, query_1.queryTableItems)(Table, {
            ...input,
            index: setIndex,
            pageLimit: 1,
            keyConditionExpression: `${hashKey} = :hashValue`,
            expressionAttributeValues: {
                [`:hashValue`]: key[hashKey],
                ...input === null || input === void 0 ? void 0 : input.expressionAttributeValues
            }
        }, dkClient);
    });
    if (!output.items[0])
        throw new Error('Not Found');
    return output.items[0];
};
exports.queryGetTableItem = queryGetTableItem;
const queryGetItem = async (KeySpace, index, keyParams, input) => {
    const item = await (0, exports.queryGetTableItem)(KeySpace.Table, index, KeySpace.indexKeyOf(index, keyParams), input, KeySpace.dkClient);
    KeySpace.assertAttributesAndKeys(item);
    return KeySpace.omitIndexKeys(item);
};
exports.queryGetItem = queryGetItem;
