import { queryTableItems } from './query';
import { run } from '../util/utils';
import { primaryIndex } from '../Table';
export const queryGetTableItem = async (Table, index, key, input, dkClient = Table.dkClient) => {
    const setIndex = index !== primaryIndex ? index : undefined;
    const hashKey = Table.config.indexes[index].hash.key;
    const sortKey = Table.config.indexes[index].sort?.key;
    const output = await run(async () => {
        if (sortKey) {
            return queryTableItems(Table, {
                ...input,
                index: setIndex,
                pageLimit: 1,
                keyConditionExpression: `${hashKey} = :hashValue AND ${sortKey} = :sortValue`,
                expressionAttributeValues: {
                    [`:hashValue`]: key[hashKey],
                    [`:sortValue`]: key[sortKey],
                    ...input?.expressionAttributeValues
                }
            }, dkClient);
        }
        return queryTableItems(Table, {
            ...input,
            index: setIndex,
            pageLimit: 1,
            keyConditionExpression: `${hashKey} = :hashValue`,
            expressionAttributeValues: {
                [`:hashValue`]: key[hashKey],
                ...input?.expressionAttributeValues
            }
        }, dkClient);
    });
    if (!output.items[0])
        throw new Error('Not Found');
    return output.items[0];
};
export const queryGetItem = async (KeySpace, index, keyParams, input) => {
    const item = await queryGetTableItem(KeySpace.Table, index, KeySpace.indexKeyOf(index, keyParams), input, KeySpace.dkClient);
    KeySpace.assertAttributesAndKeys(item);
    return KeySpace.omitIndexKeys(item);
};
