import { DkOp } from '../UpdateOp';
import { randomString } from './utils';
const createUpdateExpressionPart = (alias, key, value, precedingKeys) => ({
    updateExpression: `${precedingKeys ? `${precedingKeys.map(key => `#${key}`).join('.')}.` : ''}#${alias} = :${alias}, `,
    expressionAttributeNames: {
        [`#${alias}`]: key
    },
    expressionAttributeValues: {
        [`:${alias}`]: value
    }
});
const createUpdateExpressionParts = (attributes, precedingKeys) => {
    return Object.entries(attributes).flatMap(entry => {
        const [key, value] = entry;
        const alias = randomString(10);
        if (value instanceof DkOp) {
            const updateExpressionPart = value.createUpdateExpressionPart({ key, alias, precedingKeys });
            return [updateExpressionPart];
        }
        if (value !== null && typeof value === 'object' && !Array.isArray(value)) {
            const updateExpressionPrecedingKeyPart = {
                updateExpression: '',
                expressionAttributeNames: {
                    [`#${alias}`]: key
                },
                expressionAttributeValues: {}
            };
            return [
                updateExpressionPrecedingKeyPart,
                ...createUpdateExpressionParts(value, precedingKeys ? [...precedingKeys, alias] : [alias])
            ];
        }
        const updateExpressionPart = createUpdateExpressionPart(alias, key, value, precedingKeys);
        return [updateExpressionPart];
    });
};
export const convertObjectToUpdateExpression = (attributes) => {
    const base = {
        updateExpression: 'SET ',
        expressionAttributeNames: {},
        expressionAttributeValues: {}
    };
    const parts = createUpdateExpressionParts(attributes);
    if (parts.length === 0)
        throw new Error('Invalid update object');
    const merged = parts.reduce((accumulator, expressionPart) => {
        return {
            updateExpression: (accumulator.updateExpression || '') + expressionPart.updateExpression,
            expressionAttributeNames: {
                ...accumulator.expressionAttributeNames,
                ...expressionPart.expressionAttributeNames
            },
            expressionAttributeValues: {
                ...accumulator.expressionAttributeValues,
                ...expressionPart.expressionAttributeValues
            }
        };
    }, base);
    const updateExpression = {
        ...merged,
        updateExpression: merged.updateExpression && merged.updateExpression.length > 4
            ? merged.updateExpression.trim().slice(0, -1)
            : merged.updateExpression
    };
    return updateExpression;
};
